/* julInv.c */
/* 'Inversed' evaluations of an orbit in the Julia-set.
 */

Byte *LPICF = (Byte *) 0xFFFFFA20;
Byte *LVPW = (Byte *) 0xFFFFFA05;
ULong *LSSA=(ULong *) 0xFFFFFA00;

char *screen;
void pix(int x, int y,char c) {
 char*addr=screen+(x>>1)+(y<<4)+(y<<6);
 if(x&1) {
  *addr&=0xf0;
  *addr|=c;
 } else {
  *addr&=0x0f;
  *addr|=c<<4;
 }
}


/* 'Empirical-observation' sqrt,
 * from Pedro Freire's article 
 * http://www.pedrofreire.com/sqrt
 * Nice, kewl, and short.
 */
unsigned long sqrt_emp(unsigned long x) {
	unsigned int r; 

	x = (x+1) >> 1;           /* init x */ 
	for( r=0; x>r; x-=r++ ); 
	return r;
}

// Binary Search sqrt, from Pedro's 2nd article
unsigned long sqrt_bin(unsigned long x) {
 unsigned int r,h,sq,x2;
 sq=0x40000000;
 r=0x8000;
 i=15;
 h  = r; 
 x2 = sq; 
 do  { 
   sq >>= 2;
   h >>= 1;
   if( x >= x2 ) { 
	   x2+=(r<<i)+sq;
	   r+=h; 
   } else { 
	   x2=x2-(r<<i)+sq;
	   r-=h; 
   } 
 } while(--i); 
 if(x<=(x2-r)) 
   --r; 
 else if(x>(x2+r)) 
   ++r; 
 return r;
}

long cx,cy;

void drawJuliaInv(void) {
 cx=0;cy=0; // circle test
 long x,y;
 long accum; // for rand
 x=29<<14;
 y=23<<14; // init guesses

 accum=0x0a0f0ab0; // rand 'seed'
 for (i=0;i<6000;++i) {
  long wx,wy;
  wx=x-cx;wy=y-cy; // deduce const
  accum=(accum*(x>>10)*3632); 
 // mul-overflow rand
  
  // take complex root:
  if (wx==0&&wy==0){
   x=0;y=0;
  }else{
   long r=sqrt_emp((wx>>8)*(wx>>8)+((wy>>8)*(wy>>8)))<<8;
   if (wx>0){
    x=sqrt_emp((r+wx)>>1)<<8;
	if (x>>8) { 
		y=(wy<<7)/(x>>8);
	}
   }else{
    y=sqrt_emp((r-wx)>>1)<<8;
	if(wx<0) y=-y;
	if (y>>8){
		x=(wy<<7)/(y>>8);
	}
   }
  } // cRoot

  // choose random rootBranch:
  if (SysRandom()<sysRandomMax>>1){
 // if (accum<0x80000000)
   x=-x;y=-y;
  }
  pix(80+(x>>12),80+(y>>12),0xf);
 }
}

EventType e;
Err error;
void (*handleEvents)(void);

//mark minMaxSketch handler
void h0(void){
switch (e.eType) {
 case keyDownEvent:
   switch (e.data.keyDown.chr) {
   case pageDownChr:
 // zoom out to fullview?
     break;
   case pageUpChr:
 // zoom out x2?
     break;
   default:
     break;
   }
 break;

 case penDownEvent:
  xorSqr(minX,minY,maxX,maxY);
  minX=e.screenX; maxX=e.screenX;
  minY=e.screenY; maxY=e.screenY;
  break;

 case penMoveEvent:
  minn(minX,e.screenX);maxx(maxX,e.screenX);
  minn(minY,e.screenY);maxx(maxY,e.screenY);
  break;

 case penUpEvent:
  minn(minX,e.screenX);maxx(maxX,e.screenX);
  minn(minY,e.screenY);maxx(maxY,e.screenY);
  xorSqr(minX,minY,maxX,maxY);
  handleEvents=h1;
  break;

 default:
  break;

 }
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) {
 handleEvents=h0;
 minX=0,maxX=0,minY=0,maxY=0;
 cr0=-2<<10,ci0=-2<<10;
 dy=(3<<10)/160,dx=(3<<10)/160;
 if (cmd == sysAppLaunchCmdNormalLaunch) {
 ULong loc;
 loc =*LSSA;
 screen = MemPtrNew(12800);
 MemSet(screen, 12800, 0x00);
 *LPICF |= 0x02;
 *LVPW <<= 2;
 *LSSA =(ULong)screen;

/* draw Mand: */
if(1)
 drawJuliaInv();

 do {
  EvtGetEvent(&e, evtWaitForever);
  if (SysHandleEvent(&e))
	continue;

  handleEvents();
 } while (e.eType != appStopEvent);


 *LSSA = loc;
 *LVPW >>= 2;
 *LPICF &= 0xFC;

 }
 return 0;
}

